# from django.db import models
# from edc_model.models import BaseUuidModel
#
#
# class LimitNormal(BaseUuidModel):
#     name = models.CharField(max_length=25,)
#
#     lln_factor = models.IntegerField(default=1)
#
#     uln_factor = models.IntegerField(default=1)
#
#     class Meta:
#         constraints = ()
